import 'dart:async';

import 'package:flutter/material.dart';
import 'package:msg/msg.dart';

class StoreBuilder<Model, Part> extends StatefulWidget {
  final StoreBase<Model> store;
  final Part Function(Model model) selector;
  final Widget Function(BuildContext context, Part part) builder;

  StoreBuilder({
    required this.store,
    required this.selector,
    required this.builder,
  });

  State<StoreBuilder<Model, Part>> createState() =>
      _StoreBuilderState<Model, Part>();
}

class _StoreBuilderState<Model, Part> extends State<StoreBuilder<Model, Part>> {
  late Part part;
  late final StreamSubscription<void> rebuildSubscription;

  @override
  void initState() {
    super.initState();
    part = widget.selector(widget.store.model);
    rebuildSubscription = widget.store.stream.listen((model) {
      final newPart = widget.selector(model);
      if (newPart != part) {
        setState(() => part = newPart);
      }
    });
  }

  @override
  void dispose() {
    rebuildSubscription.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) => widget.builder(context, part);
}
